import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routines/bloc/functions_bloc/functions_bloc_bloc.dart';
import 'package:syncrow_web/pages/routines/helper/duration_format_helper.dart';
import 'package:syncrow_web/pages/routines/models/device_functions.dart';
import 'package:syncrow_web/pages/routines/models/gang_switches/switch_operational_value.dart';
import 'package:syncrow_web/pages/routines/models/water_heater/water_heater_functions.dart';
import 'package:syncrow_web/pages/routines/widgets/routine_dialogs/water_heater/water_heater_operational_values_list.dart';
import 'package:syncrow_web/utils/color_manager.dart';
import 'package:syncrow_web/utils/extension/build_context_x.dart';

class WaterHeaterValueSelectorWidget extends StatelessWidget {
  final String selectedFunction;
  final DeviceFunctionData functionData;
  final List<WaterHeaterFunctions> whFunctions;
  final AllDevicesModel? device;

  const WaterHeaterValueSelectorWidget({
    required this.selectedFunction,
    required this.functionData,
    required this.whFunctions,
    required this.device,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    final selectedFn = whFunctions.firstWhere(
      (f) => f.code == selectedFunction,
      orElse: () => WHSwitchFunction(
        deviceId: '',
        deviceName: '',
        type: '',
      ),
    );
    if (selectedFunction == 'countdown_1') {
      return Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          _buildConditionToggle(
            context,
            functionData.condition,
            selectedFunction,
            device,
            selectedFn.operationName,
            functionData,
          ),
          _buildCountDownDisplay(
            context,
            functionData.value,
            device,
            selectedFn.operationName,
            functionData,
            selectedFunction,
          ),
          _buildCountDownSlider(
            context,
            functionData.value,
            device,
            selectedFn.operationName,
            functionData,
            selectedFunction,
          ),
          const SizedBox(height: 10),
        ],
      );
    }

    return WaterHeaterOperationalValuesList(
      values: selectedFn.getOperationalValues(),
      selectedValue: functionData.value,
      device: device,
      operationName: selectedFn.operationName,
      selectCode: selectedFunction,
      onSelect: (selectedValue) async {
        context.read<FunctionBloc>().add(
              AddFunction(
                functionData: DeviceFunctionData(
                  entityId: device?.uuid ?? '',
                  functionCode: selectedFunction,
                  operationName: functionData.operationName,
                  value: selectedValue.value,
                  condition: functionData.condition,
                ),
              ),
            );
      },
    );
  }

  static Widget _buildCountDownDisplay(
      BuildContext context,
      dynamic initialValue,
      AllDevicesModel? device,
      String operationName,
      DeviceFunctionData? selectedFunctionData,
      String selectCode) {
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 20, vertical: 10),
      decoration: BoxDecoration(
        color: ColorsManager.primaryColorWithOpacity.withOpacity(0.1),
        borderRadius: BorderRadius.circular(10),
      ),
      child: Text(
        DurationFormatMixin.formatDuration(initialValue?.toInt() ?? 0),
        style: context.textTheme.headlineMedium!.copyWith(
          color: ColorsManager.primaryColorWithOpacity,
        ),
      ),
    );
  }

  static Widget _buildCountDownSlider(
    BuildContext context,
    dynamic initialValue,
    AllDevicesModel? device,
    String operationName,
    DeviceFunctionData? selectedFunctionData,
    String selectCode,
  ) {
    const twelveHoursInSeconds = 43200.0;
    final operationalValues = SwitchOperationalValue(
      icon: '',
      description: "sec",
      value: 0.0,
      minValue: 0,
      maxValue: twelveHoursInSeconds,
      stepValue: 1,
    );
    return Slider(
      value: (initialValue ?? 0).toDouble(),
      min: operationalValues.minValue?.toDouble() ?? 0.0,
      max: operationalValues.maxValue?.toDouble() ?? 0.0,
      divisions: (((operationalValues.maxValue ?? 0) -
                  (operationalValues.minValue ?? 0)) /
              (operationalValues.stepValue ?? 1))
          .round(),
      onChanged: (value) {
        context.read<FunctionBloc>().add(
              AddFunction(
                functionData: DeviceFunctionData(
                  entityId: device?.uuid ?? '',
                  functionCode: selectCode,
                  operationName: operationName,
                  value: value,
                  condition: selectedFunctionData?.condition,
                  valueDescription: selectedFunctionData?.valueDescription,
                ),
              ),
            );
      },
    );
  }

  static Widget _buildConditionToggle(
    BuildContext context,
    String? currentCondition,
    String selectCode,
    AllDevicesModel? device,
    String operationName,
    DeviceFunctionData? selectedFunctionData,
  ) {
    final conditions = ["<", "==", ">"];

    return ToggleButtons(
      onPressed: (int index) {
        context.read<FunctionBloc>().add(
              AddFunction(
                functionData: DeviceFunctionData(
                  entityId: device?.uuid ?? '',
                  functionCode: selectCode,
                  operationName: operationName,
                  condition: conditions[index],
                  value: selectedFunctionData?.value ?? 0,
                  valueDescription: selectedFunctionData?.valueDescription,
                ),
              ),
            );
      },
      borderRadius: const BorderRadius.all(Radius.circular(8)),
      selectedBorderColor: ColorsManager.primaryColorWithOpacity,
      selectedColor: Colors.white,
      fillColor: ColorsManager.primaryColorWithOpacity,
      color: ColorsManager.primaryColorWithOpacity,
      constraints: const BoxConstraints(
        minHeight: 40.0,
        minWidth: 40.0,
      ),
      isSelected:
          conditions.map((c) => c == (currentCondition ?? "==")).toList(),
      children: conditions.map((c) => Text(c)).toList(),
    );
  }
}
